@@ -3,7 +3,8 @@ globalVariables(c(
    '_tablet_N','_tablet_n', '_tablet_x',
    '_tablet_devalued_N','_tablet_devalued_n',
    '_tablet_widget', '_tablet_name', '_tablet_groupless',
-   '_tablet_stat','_tablet_groupfull','where',
+   '_tablet_stat',
+   '_tablet_groupfull','where',
    '_tablet_class'
 ))
 
@@ -687,10 +688,10 @@ groupwise <- function(x, ...)UseMethod('groupwise')
 #' @keywords internal
 #' @return class 'groupwise', arranged by groups:
 #' \item{_tablet_name}{observation identifier}
-#' \item{_tablet_level}{factor level (or special value 'numeric' for numerics)}
+#' \item{_tablet_level}{factor level or LHS of formulas in 'num'}
 #' \item{_tablet_N}{number of records}
 #' \item{_tablet_n}{number of records in group}
-#' \item{_tablet_stat}{the LHS of formulas in 'fac' and 'num'}
+# \item{_tablet_stat}{the LHS of formulas in 'fac' and 'num'}
 #' \item{_tablet_groupfull}{the LHS of formulas in 'fac' and 'num'}
 #' \item{_tablet_groupless}{the LHS of formulas in 'fac' and 'num'}
 
@@ -749,9 +750,9 @@ groupwise.data.frame <- function(
 #' example(tablet.data.frame)
 tablet <- function(x, ...)UseMethod('tablet')
 
-#' Generate a Group-wise Tablet
+#' Generate a Group-wise Tablette
 #'
-#' Generates a group-wise tablet. Calculates statistics
+#' Generates a group-wise tablette. Calculates statistics
 #' for all factors and numerics, with and without grouping
 #' variables.  Column names represent finest level of
 #' grouping, distinguished by attribute 'nest' (the values of
@@ -769,13 +770,13 @@ tablet <- function(x, ...)UseMethod('tablet')
 #' @importFrom rlang as_string
 #' @export
 #' @keywords internal
-#' @return 'tablet', with columns for each combination of groups, and:
+#' @return 'tablette', with columns for each combination of groups, and:
 #' \item{_tablet_name}{observation identifier}
 #' \item{_tablet_level}{factor level (or special value 'numeric' for numerics)}
-#' \item{_tablet_sort}{sorting column}
+# \item{_tablet_sort}{sorting column}
 #' \item{_tablet_stat}{the LHS of formulas in 'fac' and 'num'}
 #' \item{All (or value of 'all' argument)}{ungrouped results}
-tablet.groupwise <- function(
+tablette.groupwise <- function(
    x,
    ...,
    all = 'All',
@@ -903,7 +904,7 @@ tablet.groupwise <- function(
         dat <- attributes(x[[j]])
         dat <- c(dat, list(name = nm))
         out <- try(silent = TRUE, rlang::eval_tidy(rhs, data = dat))
-        if(class(out) != 'try-error'){
+        if(!inherits(out, 'try-error')){
            attr(x[[j]], 'label') <- out
         }
      }
@@ -915,14 +916,14 @@ tablet.groupwise <- function(
   # x <- mutate(x, `_tablet_sort` = as.numeric(`_tablet_level`))
   # x <- mutate(x, `_tablet_level` = as.character(`_tablet_level`))
   # x <- mutate(x, `_tablet_level` = sub('.*_tablet_', '', `_tablet_level`))
-  x$`_tablet_sort` <- as.numeric(x$`_tablet_level`)
+  #x$`_tablet_sort` <- as.numeric(x$`_tablet_level`)
   x$`_tablet_level` <- as.character(x$`_tablet_level`)
   x$`_tablet_level` <- sub('.*_tablet_', '', x$`_tablet_level`)
 
   constitutive <- c(
      '_tablet_name',
      '_tablet_level',
-     '_tablet_sort',
+    # '_tablet_sort',
      '_tablet_stat'
   )
 
@@ -934,13 +935,25 @@ tablet.groupwise <- function(
 
   # this is the only constructor for 'tablet'
   x$`_tablet_name` <- as.character(x$`_tablet_name`)
-  class(x) <- union('tablet', setdiff(class(x), 'groupwise'))
+  
+  x$`_tablet_level` <- as.character(x$`_tablet_level`)
+  x$`_tablet_stat` <- as.character(x$`_tablet_stat`)
+  x$`_tablet_level` <- ifelse(
+     x$`_tablet_level` == 'numeric',
+     x$`_tablet_stat`,
+     x$`_tablet_level`
+  )
+  x$`_tablet_stat` <- NULL
+  
+  
+  class(x) <- union('tablette', setdiff(class(x), 'groupwise'))
+  #x <- tablet(x, ...) # tablet.tablette
   x
 }
 
 #' Create Header List.
 #'
-#' Creates header list.  Generic, with method \code{\link{headerlist.tablet}}.
+#' Creates header list.  Generic, with method \code{\link{headerlist.tablette}}.
 #' @param x object
 #' @param ... passed
 #' @export
@@ -948,7 +961,7 @@ tablet.groupwise <- function(
 #' @keywords internal
 #' @examples
 #' example(classifiers)
-#' headerlist(tablet(groupwise(x)))
+#' headerlist(tablette(groupwise(x)))
 headerlist <- function(x,...)UseMethod('headerlist')
 
 #' Create Header List for Tablet
@@ -959,7 +972,7 @@ headerlist <- function(x,...)UseMethod('headerlist')
 #' @export
 #' @keywords internal
 #' @return list of named integer where each element describes an additional header row with names as indicated, repeated integer times
-headerlist.tablet <- function(x, ...){
+headerlist.tablette <- function(x, ...){
    numheaders <- 0
    out <- list()
    for(i in seq_len(ncol(x))){
@@ -995,7 +1008,7 @@ headerlist.tablet <- function(x, ...){
 
 #' Create Index
 #'
-#' Creates index.  Generic, with method \code{\link{index.tablet}}.
+#' Creates index.  Generic, with method \code{\link{index.tablette}}.
 #' @param x object
 #' @param ... passed
 #' @export
@@ -1003,7 +1016,7 @@ headerlist.tablet <- function(x, ...){
 #' @keywords internal
 #' @examples
 #' example(classifiers)
-#' index(tablet(groupwise(x)))
+#' index(tablette(groupwise(x)))
 index <- function(x,...)UseMethod('index')
 
 #' Create Index for Tablet
@@ -1015,7 +1028,7 @@ index <- function(x,...)UseMethod('index')
 #' @keywords internal
 #' @return named integer giving row groupings with names as indicated, repeated integer times
 
-index.tablet <- function(x, ...) {
+index.tablette <- function(x, ...) {
    out <- numeric(0)
    for (row in seq_len(nrow(x))) {
       r <- as.character(x$`_tablet_name`)[[row]]
@@ -1047,7 +1060,7 @@ index.tablet <- function(x, ...) {
 #' @keywords internal
 #' @examples
 #' example(classifiers)
-#' as_kable(tablet(groupwise(x)))
+#' as_kable(tablet(tablette(groupwise(x))))
 as_kable <- function(x, ...)UseMethod('as_kable')
 # https://colinfay.me/writing-r-extensions/generic-functions-and-methods.html
 # https://stackoverflow.com/questions/13984470/possible-to-create-latex-multicolumns-in-xtable
@@ -1151,7 +1164,7 @@ as_kable.tablet <- function(
    linebreaker = '\n',
    pack_rows = list(escape = escape)
 ){
-
+   x <- tablette(x, ...)
    # if(is.na(escape)){
    #    if (knitr::is_latex_output()){
    #      escape <- FALSE
@@ -1160,12 +1173,12 @@ as_kable.tablet <- function(
    #    }
    # }
    stopifnot(is.logical(escape), length(escape) == 1)
-   x$`_tablet_sort` <- NULL
+   # x$`_tablet_sort` <- NULL
    index <- index(x)
    # draws on _tablet_name, which should be character or c('latex', 'character')
    nmsi <- names(index) # isolate to assign class
    stopifnot(is.character(x$`_tablet_name`))
-   class(nmsi) <- class(x$`_tablet_name`) # class propagaton
+   class(nmsi) <- class(x$`_tablet_name`) # class propagation
    x$`_tablet_name` <- NULL # done
    if(!escape){
       if (knitr::is_latex_output()) {
@@ -1177,14 +1190,13 @@ as_kable.tablet <- function(
    # nmsi now as informed as possible ... assign back
    names(index) <- nmsi
    #x$`_tablet_level` <- as.character(x$`_tablet_level`)
-   x$`_tablet_stat` <- as.character(x$`_tablet_stat`)
-   #x <- mutate(x, `_tablet_level` = ifelse(`_tablet_level` == 'numeric', `_tablet_stat`, `_tablet_level`))
-   x$`_tablet_level` <- ifelse(
-      x$`_tablet_level` == 'numeric',
-      x$`_tablet_stat`,
-      x$`_tablet_level`
-   )
-   x$`_tablet_stat` <- NULL
+   # x$`_tablet_stat` <- as.character(x$`_tablet_stat`)
+   # x$`_tablet_level` <- ifelse(
+   #    x$`_tablet_level` == 'numeric',
+   #    x$`_tablet_stat`,
+   #    x$`_tablet_level`
+   # )
+   # x$`_tablet_stat` <- NULL
    # names(x)[names(x) == 'level'] <- ''
    headerlist <- headerlist(x)
    for(i in seq_len(ncol(x))){
@@ -1293,11 +1305,11 @@ as_kable.tablet <- function(
 #' NA, the values of arguments beginning with 'na.rm' or 'exclude'
 #' may not matter.
 #'
-#' Output includes the column \code{_tablet_name} which inherits character.
+#' Column 1 of output is character.
 #' Its values are typically the names of the original columns
 #' that were factor or numeric but not in groups(x). If the first
 #' of these had a label attribute or (priority) a title attribute
-#' with class 'latex', then \code{_tablet_name} is assigned the
+#' with class 'latex', then column 1 is assigned the
 #' class 'latex' as well. It makes sense therefore to be consistent
 #' across input columns regarding the presence or not of a 'latex'
 #' label or title. By default, \code{\link{as_kable.tablet}} dispatches
@@ -1349,13 +1361,26 @@ as_kable.tablet <- function(
 #' @param all_levels whether to supply records for unobserved levels
 #' @importFrom dplyr all_of across everything full_join anti_join
 #' @importFrom magrittr %>% %<>%
+#' @importFrom yamlet as_dvec
 #' @export
-#' @return 'tablet', with columns for each combination of groups, and:
-#' \item{_tablet_name}{observation identifier: character, possibly 'latex', see details; has a codelist attribute the values of which are the original column names}
-#' \item{_tablet_level}{factor level (or special value 'numeric' for numerics)}
-#' \item{_tablet_stat}{the LHS of formulas in 'fac' and 'num'}
-#' \item{All (or value of 'all' argument)}{ungrouped results}
-#' \item{_tablet_sort}{sorting column}
+#' @return 'tablet'
+#' A tablet is a special case of data.frame with grouped rows and columns.
+#' \item{*}{There is always one level of row groups.}
+#' \item{*}{There can be any number of column groups, including zero.}
+#' \item{*}{All columns are character (as tested by \code{is.character()}).}
+#' \item{*}{The first column has empty strings that represent
+#' the last non-empty value. It can be class 'latex' or 'character'.}
+#' \item{*}{Leading element(s) of first column are deliberately blank (one space character)
+#' and correspond to header rows.  See \code{\link{header_rows}}.}
+#' \item{*}{The second column represents group-specific property 
+#' names. It is populated always and only where column 1 is not.}
+#' \item{*}{All other columns represent group-specific property values; 
+#' elements before the first non-empty group value represent nested headers.}
+#' \item{*}{Header values may be repeated.}
+#' \item{*}{Header values may be empty strings, representing the last non-empty value
+#' to the left, or single spaces, which are deliberately blank.}
+#' \item{*}{Internally, character NA is equivalent to an empty string.}
+#' 
 #' @seealso \code{\link{as_kable.tablet}}
 #' @examples
 #' library(boot)
@@ -1397,7 +1422,7 @@ tablet.data.frame <- function(
    exclude_name = NULL,
    all_levels = FALSE
 ){
-   y <- groupwise( # groupwise.data.frame
+   y <- groupwise( # groupwise.data.frame, returns tablette
       x,
       ...,
       na.rm = na.rm,
@@ -1412,7 +1437,7 @@ tablet.data.frame <- function(
       exclude_name = exclude_name,
       all_levels = all_levels
    )
-   y <- tablet(y, ..., all = all, lab = lab ) # tablet.groupwise
+   y <- tablette(y, ..., all = all, lab = lab ) # tablette.groupwise
    y$`_tablet_name` <- as.character(y$`_tablet_name`)
 
    codes <- unique(y$`_tablet_name`)
@@ -1442,6 +1467,16 @@ tablet.data.frame <- function(
             class(y$`_tablet_name`) <- union('latex', class(y$`_tablet_name`))
       }
    }
+   # y$`_tablet_level` <- as.character(y$`_tablet_level`)
+   # y$`_tablet_stat` <- as.character(y$`_tablet_stat`)
+   # y$`_tablet_level` <- ifelse(
+   #    y$`_tablet_level` == 'numeric',
+   #    y$`_tablet_stat`,
+   #    y$`_tablet_level`
+   # )
+   # y$`_tablet_stat` <- NULL
+  # x$`_tablet_sort` <- NULL
+   y <- tablet(y, ...)
    y
 }
 
@@ -1492,7 +1527,7 @@ splice.data.frame <- function(x, all = 'All', ...){
       for(i in setdiff(grp, list(group))) this[i] <- NULL
       tot <- all
       if(g < length(grp)) tot <- character(0) # 'all' only for last table
-      that <- tablet(this, all = tot, ...)
+      that <- tablette(tablet(this, all = tot, ...))
       part[[group]] <- that
    }
    names(part) <- NULL
@@ -1510,6 +1545,7 @@ splice.data.frame <- function(x, all = 'All', ...){
    }
    out <- out[,!dupcol, drop = FALSE]
    class(out) <- class(part[[1]])
+   out <- tablet(out)
    out
 }
 
